package com.jiuqi.grid;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.jiuqi.grid.CellDataProcessorIntf;
import com.jiuqi.grid.CellDataProperty;
import com.jiuqi.grid.CellDataPropertyIntf;
import com.jiuqi.grid.GridCell;
import com.jiuqi.grid.NumberCellPropertyIntf;
import com.jiuqi.util.StringUtils;
import com.jiuqi.util.chinese.ChineseDouble;
import java.io.Serializable;
import java.text.DecimalFormat;

public class NumberCellProperty implements NumberCellPropertyIntf, CellDataProcessorIntf, Serializable {
  protected CellDataPropertyIntf cellDataProperty;
  
  public NumberCellProperty(CellDataPropertyIntf dataProperty) {
    this.cellDataProperty = dataProperty;
  }
  
  public NumberCellProperty(GridCell cell) {
    this((CellDataPropertyIntf)new CellDataProperty(cell));
  }
  
  public CellDataPropertyIntf getCellDataProperty() {
    return this.cellDataProperty;
  }
  
  public int getDecimal() {
    return this.cellDataProperty.getDataFlag();
  }
  
  public void setDecimal(int value) {
    if (value <= 15) {
      if (value < 0)
        value = 0; 
      this.cellDataProperty.setDataFlag((byte)value);
    } else {
      throw new IllegalArgumentException("错误设置了数字的小数位数，应该在0-15之间！");
    } 
  }
  
  public int getPrecision() {
    return this.cellDataProperty.getDataProperty() & 0x1F;
  }
  
  public void setPrecision(int value) {
    if (value >= 0 && value <= 32) {
      int data = value & 0x1F;
      this.cellDataProperty.setDataProPerty(
          
          (short)(this.cellDataProperty.getDataProperty() & 0xFFFFFFE0 | data));
    } else {
      throw new IllegalArgumentException("错误设置了数字的精度，应该在0-32之间！");
    } 
  }
  
  public boolean getThoundsMark() {
    return ((this.cellDataProperty.getDataProperty() & 0x8000) != 0);
  }
  
  public void setThoundsMarks(boolean value) {
    if (value) {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() | 0x8000));
    } else {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() & 0xFFFF7FFF));
    } 
  }
  
  public boolean getBracketNegative() {
    return ((this.cellDataProperty.getDataProperty() & 0x4000) != 0);
  }
  
  public void setBracketNegative(boolean value) {
    if (value) {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() | 0x4000));
    } else {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() & 0xFFFFBFFF));
    } 
  }
  
  public boolean getWarningNegative() {
    return ((this.cellDataProperty.getDataProperty() & 0x2000) != 0);
  }
  
  public void setWarningNegative(boolean value) {
    if (value) {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() | 0x2000));
    } else {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() & 0xFFFFDFFF));
    } 
  }
  
  public boolean getChineseNumber() {
    return ((this.cellDataProperty.getDataProperty() & 0x1000) != 0);
  }
  
  public void setChineseNumber(boolean value) {
    if (value) {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() | 0x1000));
    } else {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() & 0xFFFFEFFF));
    } 
  }
  
  public boolean getBigChineseChar() {
    return ((this.cellDataProperty.getDataProperty() & 0x800) != 0);
  }
  
  public void setBigChineseChar(boolean value) {
    if (value) {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() | 0x800));
    } else {
      this.cellDataProperty.setDataProPerty(
          (short)(this.cellDataProperty.getDataProperty() & 0xFFFFF7FF));
    } 
  }
  
  public boolean getIsPercent() {
    if (this.cellDataProperty.getDataType() == 2)
      return ((this.cellDataProperty.getDataProperty() & 0x400) != 0); 
    return false;
  }
  
  public void setIsPercent(boolean value) {
    if (this.cellDataProperty.getDataType() == 2)
      if (value) {
        this.cellDataProperty.setDataProPerty(
            (short)(this.cellDataProperty.getDataProperty() | 0x400));
      } else {
        this.cellDataProperty.setDataProPerty(
            (short)(this.cellDataProperty.getDataProperty() & 0xFFFFFBFF));
      }  
  }
  
  public String getEditText(String storeText) {
    return storeText;
  }
  
  public String getShowText(String storeText) {
    if (storeText != null) {
      String showText = null;
      if (storeText == "") {
        showText = "";
      } else {
        storeText = StringUtils.eliminate(storeText, ',');
        try {
          value = Double.parseDouble(storeText);
        } catch (NumberFormatException numberFormatException) {
          return storeText;
        } 
        boolean isNegtive = (value < 0.0D);
        boolean isPercent = getIsPercent();
        boolean showThoundsMark = getThoundsMark();
        if (isNegtive)
          value = -value; 
        if (isPercent && !showThoundsMark) {
          value *= 100.0D;
          setDecimal(getDecimal() - 2);
        } else if (isPercent && showThoundsMark) {
          value *= 1000.0D;
          setDecimal(getDecimal() - 3);
        } 
        int decimal = getDecimal();
        if (decimal == 15)
          decimal = 0; 
        double value = trimDouble(value, getDecimal());
        if (!getChineseNumber()) {
          showText = formatNumberString(value, getDecimal(), 
              getThoundsMark());
          if (isPercent && !showThoundsMark)
            showText = String.valueOf(showText) + "%"; 
          if (isPercent && showThoundsMark)
            showText = String.valueOf(showText) + "‰"; 
          if (isNegtive)
            if (getBracketNegative()) {
              showText = "(" + showText + ")";
            } else {
              showText = "-" + showText;
            }  
        } else {
          if (isPercent && !showThoundsMark) {
            ChineseDouble cnDouble = new ChineseDouble(value / 100.0D);
            showText = cnDouble.getPercentForm(getDecimal(), 
                true, getBigChineseChar());
          } 
          if (isPercent && showThoundsMark) {
            ChineseDouble cnDouble = new ChineseDouble(value / 1000.0D);
            showText = cnDouble.getPermillageForm(
                getDecimal(), true, getBigChineseChar());
          } 
          if (!isPercent) {
            ChineseDouble cnDouble = new ChineseDouble(value);
            if (!showThoundsMark) {
              showText = cnDouble.chineseValue(getDecimal(), 
                  false, getBigChineseChar());
            } else {
              showText = cnDouble.chineseValue(getDecimal(), 
                  true, getBigChineseChar());
            } 
          } 
          if (isNegtive)
            showText = "（负）" + showText; 
        } 
        if (isPercent)
          setDecimal(getDecimal() + (showThoundsMark ? 3 : 2)); 
      } 
      return showText;
    } 
    throw new IllegalArgumentException("不能解析指向null的字符串为数字！");
  }
  
  public String parseEditText(String editText) {
    double value = Double.parseDouble(editText);
    return formatNumberString(value, getDecimal(), false);
  }
  
  public String parseShowText(String showText) {
    String value = "";
    if (!getChineseNumber()) {
      value = showText;
    } else {
      try {
        value = ChineseDouble.valueOf(showText);
      } catch (Exception exception) {
        throw new IllegalArgumentException("中文数值有错误，解析失败");
      } 
    } 
    double dbValue = Double.parseDouble(value);
    return formatNumberString(dbValue, getDecimal(), 
        getThoundsMark());
  }
  
  private static double trimDouble(double source, int frcLength) {
    if (frcLength < 0)
      frcLength = 0; 
    double des = source;
    int multiplier = (int)Math.exp(Math.log(10.0D) * frcLength);
    double temp = Math.round(source * multiplier);
    des = temp / multiplier;
    return des;
  }
  
  private static String formatNumberString(double value, int decimal, boolean thoundsMark) {
    DecimalFormat df = new DecimalFormat();
    df.setGroupingUsed(false);
    if (thoundsMark) {
      df.setGroupingSize(3);
      df.setGroupingUsed(true);
    } 
    if (decimal >= 0) {
      df.setMaximumFractionDigits(decimal);
      df.setMinimumFractionDigits(decimal);
    } 
    return df.format(value);
  }
}
